# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)

# CCI Progress
progress.rda <- file.path(parent.dir, "Prepared Data", "CCI Progress.rda")
load(progress.rda)
head(cai_totals)

# CCI CAIs by Category
breakdown.rda <- file.path(parent.dir, "Prepared Data",
                           "CCI CAIs by Category.rda")
load(breakdown.rda)
head(connected_by_category)

# CCI Speed and Pricing
prspd.rda <- file.path(parent.dir, "Prepared Data", 
                       "CCI Speed and Pricing.rda")
load(prspd.rda)
head(price_speed)



# Table. Annual Extrapolated CAI Pricing and Speed Change Benefits ####

# Differences in median speed and price
d <- copy(price_speed)
# Only summarize those with before and after values
d[is.na(price_mbps_before), price_mbps_after := NA_real_]
d[is.na(price_mbps_after), price_mbps_before := NA_real_]
d[is.na(speed_after), speed_before := NA_real_]
d[is.na(speed_before), speed_after := NA_real_]
# Summarize
med_diff <- d[, lapply(.SD, median, na.rm = TRUE), keyby = category, 
              .SDcols = c("speed_before", "price_mbps_before", "speed_after", 
                          "price_mbps_after")]
# Public safety missing price data, use overall medians
med_diff["Public Safety", c("price_mbps_before", "price_mbps_after") := d[
  , lapply(.SD, median, na.rm = TRUE), .SDcols = c("price_mbps_before",
                                                   "price_mbps_after")]]
# ASR adjustment: cap Other Community Support and Public Safety after speeds at
# 100 Mbps - median of schools, healthcare, and libraries
d[category %in% c("School (K-12)", "Medical/Healthcare", "Library"),
  median(speed_after)]
med_diff[category %in% c("Other Community Support", "Public Safety"), 
         speed_after := 100]
# Differences
med_diff[, speed_change := speed_after - speed_before]
med_diff[, price_mbps_change := price_mbps_before - price_mbps_after]
# Per-CAI annual benefit
med_diff[, benefit := speed_change * price_mbps_change * 12]

# Estimate number of CAIs in each category
# Don't included defunded and excluded
total_cais <- grants[cai_totals][Defunded == "" & Excluded == "",
                                 sum(ComInst_Total_Num)]
percentages <- connected_by_category[, list(connected = sum(connected)), 
                                     keyby = category]
percentages[, percentage := connected / sum(connected)]
category_cais <- percentages[, list(
  category, percentage, connected = percentage * total_cais)]
setkey(category_cais, category)

# Merge counts and benefits
category_cais[med_diff, benefit := benefit]

# Extrapolate total annual benefit
category_cais[, total_benefit := (connected * benefit)]

# Total
tot <- category_cais[, lapply(.SD, sum), .SDcols = 2:5, by = list(
  category = rep("All institutions", nrow(category_cais)))]
tot[, benefit := total_benefit / connected]
category_cais <- rbind(category_cais, tot)

# Reorder, rename, and format
comma <- function(x, digits = 0) formatC(x, format = "f", digits = digits,
                                         big.mark = ",")
(extrap_pricespeed_benefits <- category_cais[, list(
  `Institution Type` = category, 
  `Percent of All CAIs` = sprintf("%.0f%%", percentage * 100), 
  `Number of CAIs` = comma(connected), 
  `Total Cost Difference (Millions)` = paste0("$", comma(total_benefit / 1e6)), 
  `Per CAI Cost Difference` = paste0("$", comma(benefit)))])

# Write
write.xlsx(extrap_pricespeed_benefits, file.path(sec4, "Table 14.xlsx"),
           row.names = FALSE)
