# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load analysis ####

# Extrapolation Table
extrapolate.rda <- file.path(parent.dir, "Statistical Analysis", 
                             "Extrapolation Table.rda")
load(extrapolate.rda)
head(extrapolate)



# Load Prepared Data ####

# Gross County Product
gcp.rda <- file.path(parent.dir, "Prepared Data", "Gross County Product.rda")
load(gcp.rda)
head(gcp)

# NBM Population Statistics
nbm.population.rda <- file.path(parent.dir, "Prepared Data",
                                "NBM Population Statistics.rda")
load(nbm.population.rda)
population <- availability[, list(fips, population = pop_total_2011_06)]
setkey(population, fips)
head(population)



# Czernich et al, Broadband Infrastructure and Economic Growth ####
# The introduction of broadband increases raises GDP per capita by 2.7 to 3.9 
# percent

# Lookup
czernich <- copy(extrapolate) # BTOP counties
czernich <- gcp[czernich] # gross county product
czernich <- population[czernich] # population

# Some (in HI and PR) missing GCP
# Per capita GCP
czernich[, pc_gcp := gcp / population]
# Median of nonmissing per capita GCP
czernich[, med_pc_gcp := median(pc_gcp, na.rm = TRUE)]
# If missing GCP, replace with median GCP * population
czernich[is.na(gcp), gcp := population * med_pc_gcp]

# Project economic benefit - the increase in GDP
czernich[, economic_benefit := (effect * .027 * gcp) / 1e6]

# Print totals
czernich[, list(benefit = sum(economic_benefit)), keyby = list(
  definition, adjustment, match.type)]



# LECG Ltd, Economic Impact of Broadband ####
# The econometric model that we have developed predicts that an increase of 1 
# percentage point 1 more broadband line per 100 individuals) in these "medium 
# or high ICT" countries increases productivity by 0.1%.

# Lookup  
lecg <- copy(extrapolate) # BTOP counties
lecg <- gcp[lecg] # gross county product
lecg <- population[lecg] # population

# Some (in HI and PR) missing GCP
# Per capita GCP
lecg[, pc_gcp := gcp / population]
# Median of nonmissing per capita GCP
lecg[, med_pc_gcp := median(pc_gcp, na.rm = TRUE)]
# If missing GCP, replace with median GCP * population
lecg[is.na(gcp), gcp := population * med_pc_gcp]

# Project economic benefit
lecg[, economic_benefit := (effect * .1 * gcp) / 1e6]

# Print totals
lecg[, list(benefit = sum(economic_benefit)), keyby = list(
  definition, adjustment, match.type)]



# Results ####

# Bind together
inc_gdp <- rbind(czernich[, Study := "Czernich et al. (2011)"],
                 lecg[, Study := "LECG Ltd. (2009)"])

# Aggregate table
(inc_gdp <- tabular(
  Factor(Study) * Heading("Grants") * (case + Heading("All BTOP") * 1) * 
    Heading("Match Type") * match.type ~
    Heading() * adjustment * Heading() * definition * Heading() *
    economic_benefit * Heading() * sum * 
    Format(formatC(digits = 0, format = "f", big.mark = ",")), inc_gdp))

# Write
write.xlsx.tabular(inc_gdp, file.path(
  appD, 
  "D.8 Long-Term Impact of BTOP CCI Infrastructure on GDP - Table 55.xlsx"))
