# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Prepared Data ####

# NBM Population Statistics
nbm.rda <- file.path(parent.dir, "Prepared Data", 
                     "NBM Population Statistics.rda")
load(nbm.rda)
head(availability)



# Input files ####

# Service areas
asr.cases.file <- file.path(parent.dir, "Input Files", 
                            "CCI Evaluation Study Sample Service Areas.xlsx")
grant.service.areas <- Read.xlsx2(asr.cases.file, sheetIndex = 1)
setkey(grant.service.areas, fips)

# County attributes
county.att.file <- file.path(
  parent.dir, "Input Files",
  "Census Bureau County Shapefiles Selected Attributes.xlsx")
county.att <- Read.xlsx2(county.att.file, sheetIndex = 1, colClasses = c(
  rep("character", 2), "numeric"))
setkey(county.att, fips)

# Convert land area from square meters to square miles
county.att[, sq_miles := ALAND10 * 3.86102e-7]

# Make sub out county/parish/municipio from county name
county.att[, county := gsub("\\s$", "", gsub(
  "Municipio", "", gsub("Parish", "", gsub("County", "", NAMELSAD10))))]


# Grant info ####

# Add column of short names with state for grants
grantees <- c(
  "South Dakota Network (South Dakota)" = "NT10BIX5570003",
  "Merit Network, Inc. (Michigan)" = "NT10BIX5570009",
  "Mid-Atlantic Broadband Communities Corporation (Virginia)" =
    "NT10BIX5570019",
  "Zayo Bandwidth (Indiana)" = "NT10BIX5570025",
  "Executive Office of the State of West Virginia (West Virginia)" = 
    "NT10BIX5570031",
  "Lane Council of Governments (Oregon)" = "NT10BIX5570032",
  "OneCommunity (Ohio)" = "NT10BIX5570067",
  "Massachusetts Technology Park (Massachusetts)" = "NT10BIX5570070",
  "Clearwave Communications (Illinois)" = "NT10BIX5570100",
  "University of Arkansas System (Arkansas)" = "NT10BIX5570102",
  "MCNC (North Carolina)" = "NT10BIX5570120",
  "OSHEAN (Rhode Island)" = "NT10BIX5570153")
grantees <- grantees[order(names(grantees))]
grant.service.areas[, grantee := factor(Award_Num, levels = grantees, 
                                        labels = names(grantees))]

# Add on name and land area
grant.service.areas[county.att, `:=`(county = county, sq_miles = sq_miles)]

# Add on population and population without NTIA unadjusted availability
grant.service.areas[availability, `:=`(
  population = pop_total_2011_06, 
  pop_nobb = pop_total_2011_06 - pop_bb_ntia_2011_06)]

# One multi-state
grant.service.areas["25005", `:=`(county = paste0(county, ", Massachusetts"),
                                  out.of.state = TRUE)]
grant.service.areas[is.na(out.of.state), out.of.state := FALSE]

# Collapse rows into lists of counties
grant.service.areas <- grant.service.areas[
  order(grantee, out.of.state, added_cai, county), list(
    counties = paste(county, collapse = ", "), .N, sq_miles = sum(sq_miles), 
    population = sum(population), pop_nobb = sum(pop_nobb)), 
  keyby = list(grantee, out.of.state, added_cai)]
grant.service.areas[added_cai == TRUE, counties := paste(
  "Added due to connected CAI:", counties)]
grant.service.areas <- grant.service.areas[, list(
  counties = paste(counties, collapse = "; and "), N = sum(N), 
  sq_miles = sum(sq_miles), population = sum(population), 
  pop_nobb = sum(pop_nobb)), keyby = list(grantee, added_cai)]
grant.service.areas <- grant.service.areas[, list(
  counties = paste(counties, collapse = "\r\n\r\n"), N = sum(N), 
  sq_miles = sum(sq_miles), population = sum(population), 
  pop_nobb = sum(pop_nobb)), keyby = grantee]


# Add totals ####
total <- data.table("Total", "", grant.service.areas[
  , lapply(.SD, sum), .SDcols = c("N", "sq_miles", "population", "pop_nobb")])
setnames(total, names(total), names(grant.service.areas))
grant.service.areas <- rbind(grant.service.areas, total)


# Format ####
sdc <- c("sq_miles", "population", "pop_nobb")
grant.service.areas[
  , sdc := lapply(.SD, formatC, digits = 0, format = "f", big.mark = ","), 
  .SDcols = sdc, with = FALSE]


# Rename ####
setnames(grant.service.areas, names(grant.service.areas), c(
  "Grant (State)", "Counties", "Number of Counties", "Land Area (mi2)", 
  "Population (June 30, 2011)", 
  "Population without Availability (June 30, 2011)"))


# Write to file ####
service.xlsx <- file.path(appD, "Table 46.xlsx")
write.xlsx2(grant.service.areas, service.xlsx, row.names = FALSE)
