# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code",
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"),
       echo = TRUE)



# Load Prepared Data ####

# NBM Population Statistics
nbm.rda <- file.path(parent.dir, "Prepared Data",
                     "NBM Population Statistics.rda")
load(nbm.rda)
head(availability)



# Block level populations and broadband ####
block.file <- file.path(
  parent.dir, "Input Files", 
  "NBM Census Blocks with Availability Rate Decreases by County.xlsx")
block_level <- Read.xlsx2(block.file, 1, colClasses = c(
  "character", "logical", rep("integer", 7)))

# How many decreases?
blocks <- block_level[, lapply(.SD, sum), .SDcols = 3:9]
pop_blocks <- block_level[(populated), lapply(.SD, sum), .SDcols = 3:9]



# County level populations and broadband ####

# From Prepared Data
county_level <- availability[as.numeric(substr(fips, 1, 2)) %in% 
                               setdiff(1:56, c(2, 15))]

## Change over time (a positive number will indicate a decrease)
county_level[, dec_nofa_amt := ppop_bb_nofa_2011_06 - ppop_bb_nofa_2013_06]
county_level[, dec_ntia_amt := ppop_bb_ntia_2011_06 - ppop_bb_ntia_2013_06]

## Decrease size indicators
ind <- c("dec_nofa_amt", "dec_ntia_amt")
county_level[, paste0(ind, "_0") := lapply(.SD, ">",  0.00), .SDcols = ind]
county_level[, paste0(ind, "_1") := lapply(.SD, ">=", 0.01), .SDcols = ind]
county_level[, paste0(ind, "_5") := lapply(.SD, ">=", 0.05), .SDcols = ind]

## How many decreases?
county_level[, all := 1L]
counties <- county_level[, lapply(.SD, sum), .SDcols = c(
  "all", paste0(ind, "_0"), paste0(ind, "_1"), paste0(ind, "_5"))]



# Summarize results

## Reshape and combine
reshaped <- rbind(melt(blocks, id.vars = "all")[, set := 1], 
                  melt(pop_blocks, id.vars = "all")[, set := 2], 
                  melt(counties, id.vars = "all")[, set := 3])

## Make prettier labels
reshaped[, variable := as.character(variable)]
reshaped[, Definition := factor(toupper(substr(variable, 5, 8)))]
reshaped[, Description := substring(variable, nchar(variable))] 
reshaped[, adjustment := gsub("[_0-9]", "", substring(variable, 13))]
reshaped[, adjustment := factor(adjustment, c("", "forw", "back"), c(
  "Availability", "Forward Looking", "Backward Looking"))]
reshaped[, Description := factor(Description, paste(c(0, 1, 5)), c(
  "Decrease in availability rate", "Decrease in availability rate >= 1%",
  "Decrease in availability rate >= 5%"))]
reshaped[, set := factor(set, 1:3, c("Census Blocks", "Populated Blocks",
                                     "Counties"))]
setnames(reshaped, "value", "N")

## Calculate percentage of total
reshaped[, `%` := N / all * 100]



# Table 50. Census Blocks and Counties with Reported Decreases in the Broadband
# Availability Rate over the Study Period ####
(cblock_count_dec <- tabular(
  Definition * Factor(Description, texify = FALSE) ~ Heading() * set * (
    N * Format(formatC(digits = 0, big.mark = ",", format = "f")) + `%` * 
      Format(sprintf("%.2f%%"))) * Heading() * sum, 
  reshaped[adjustment == "Availability"]))
write.xlsx.tabular(cblock_count_dec, file.path(appD, "Table 48.xlsx"))
