# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code",
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# NBM Population Statistics
nbm.rda <- file.path(parent.dir, "Prepared Data", 
                     "NBM Population Statistics.rda")
load(nbm.rda)
head(availability)



# County level populations and broadband ####

# From Prepared Data
county_level <- availability[as.numeric(substr(fips, 1, 2)) %in% 
                               setdiff(1:56, c(2, 15))]

## Change over time (a positive number will indicate a decrease)
county_level[, `:=`(
  dec_nofa_amt = ppop_bb_nofa_2011_06 - ppop_bb_nofa_2013_06,
  dec_nofa_amt_back = ppop_bb_nofa_2011_06 - ppop_bbbackward_nofa_2013_06,
  dec_nofa_amt_forw = ppop_bbforward_nofa_2011_06 - ppop_bb_nofa_2013_06,
  dec_ntia_amt = ppop_bb_ntia_2011_06 - ppop_bb_ntia_2013_06,
  dec_ntia_amt_back = ppop_bb_ntia_2011_06 - ppop_bbbackward_ntia_2013_06,
  dec_ntia_amt_forw = ppop_bbforward_ntia_2011_06 - ppop_bb_ntia_2013_06)]

## Decrease size indicators
ind <- c("dec_nofa_amt", "dec_ntia_amt", "dec_nofa_amt_back", 
         "dec_ntia_amt_back", "dec_nofa_amt_forw", "dec_ntia_amt_forw")
county_level[, paste0(ind, "_0") := lapply(.SD, ">",  0.00), .SDcols = ind]
county_level[, paste0(ind, "_1") := lapply(.SD, ">=", 0.01), .SDcols = ind]
county_level[, paste0(ind, "_5") := lapply(.SD, ">=", 0.05), .SDcols = ind]

## How many decreases?
county_level[, all := 1L]
counties <- county_level[, lapply(.SD, sum), .SDcols = c(
  "all", paste0(ind, "_0"), paste0(ind, "_1"), paste0(ind, "_5"))]



# Summarize results

## Reshape and combine
reshaped <- melt(counties, id.vars = "all")[, set := 3]

## Make prettier labels
reshaped[, variable := as.character(variable)]
reshaped[, Definition := factor(toupper(substr(variable, 5, 8)))]
reshaped[, Description := substring(variable, nchar(variable))] 
reshaped[, adjustment := gsub("[_0-9]", "", substring(variable, 13))]
reshaped[, adjustment := factor(adjustment, c("", "forw", "back"), c(
  "Availability", "Forward Looking", "Backward Looking"))]
reshaped[, Description := factor(Description, paste(c(0, 1, 5)), c(
  "Decrease in availability rate", "Decrease in availability rate >= 1%",
  "Decrease in availability rate >= 5%"))]
reshaped[, set := factor(set, 1:3, c("Census Blocks", "Populated Blocks", 
                                     "Counties"))]
setnames(reshaped, "value", "N")

## Calculate percentage of total
reshaped[, `%` := N / all * 100]



# Table. Counties with Reported Decreases in the Broadband Availability Rate
# (Unadjusted and Adjusted) over the Study Period ####
(adj_count_dec <- tabular(
  Definition * Factor(Description, texify = FALSE) ~ 
    Heading() * adjustment * (N * Format(
      formatC(digits = 0, big.mark = ",", format = "f")) + `%` * 
        Format(sprintf("%.2f%%"))) * Heading() * sum, 
  reshaped[set == "Counties"]))
write.xlsx.tabular(adj_count_dec, file.path(appD, "Table 49.xlsx"))
