# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load analysis ####

# Matching
matching.rda <- file.path(parent.dir, "Statistical Analysis", "Matching.rda")
load(matching.rda)
head(match.data)



# Match results ####

# List different matches
match.list <- c("m.nofa.back.prim", "m.nofa.back.sens", 
                "m.nofa.forw.prim", "m.nofa.forw.sens",
                "m.nofa.unad.prim", "m.nofa.unad.sens",
                "m.ntia.back.prim", "m.ntia.back.sens", 
                "m.ntia.forw.prim", "m.ntia.forw.sens", 
                "m.ntia.unad.prim", "m.ntia.unad.sens")

# Combine results into a list
matches <- mget(match.list, .GlobalEnv)

# Which availability rates and covariates were used?
avail <- lapply(matches, attr, "availability")
covar <- lapply(matches, attr, "covariates")

# Combine availability rates, population, and covariates used for each
var <- Map("c", avail, covar)



# Table 52. Descriptive Statistics of Matching Variables ####

# Which variables do we want to summarize? (all possible availability rates, population,
# and all possible covariates)
llev <- c(
  "ppop_bb_nofa_2011_06", "ppop_bb_ntia_2011_06", "ppop_bbforward_nofa_2011_06",
  "ppop_bbforward_ntia_2011_06", "ppop_bbbackward_nofa_2011_06", 
  "ppop_bbbackward_ntia_2011_06", "ppop_rural_2011_06", "ppop_minority_2011_06", 
  "ppop_over60_2011_06", "ppop_poverty_2011_06", "non_english")
llab <- c(
  "Availability (NOFA)", "Availability (NTIA)", 
  "Forward Looking Availability (NOFA)", "Forward Looking Availability (NTIA)", 
  "Backward Looking Availability (NOFA)",
  "Backward Looking Availability (NTIA)", "Rurality", "Minority", 
  "Over 60 Years Old", "Poverty", "Non-English")

# Extract treatment and control data from match data table
setkey(match.data, fips)
treat <- Map(function(m, v) match.data[m, v, with = FALSE][, tc := 1], 
             lapply(matches, setkeyv, "treat"), var)
control <- Map(function(m, v) match.data[m, v, with = FALSE][, tc := 2], 
               lapply(matches, setkeyv, "control"), var)

# Combine treatment and control data
Rbind.fill <- function(...) as.data.table(rbind.fill(...))
treat <- Rbind.fill(treat)[, v := unlist(Map(rep, names(treat), 
                                             lapply(treat, nrow)))]
control <- Rbind.fill(control)[, v := unlist(Map(rep, names(control), 
                                                 lapply(control, nrow)))]
d <- Rbind.fill(treat, control)

# Reshape to long
d <- melt(d, id.vars = c("tc", "v"), variable.name = "var")[!is.na(value)]

# Get mean and median statistics
d <- d[, list(Mean = mean(value), Median = median(value)), 
       by = list(tc, v, var)]

# Reshape to long
d <- melt(d, id.vars = 1:3, variable.name = "Statistic")

# Add factor variables to use for summary table
d[, tc := factor(tc, labels = c("Treatment", "Control"))]
d[, variable := droplevels(factor(var, llev, llab))]
levels(d$variable)[1:6] <- "Availability"
d[, Definition := factor(grepl("nofa", v), c(TRUE, FALSE), c("NOFA", "NTIA"))]
d[, Adjustment := factor(sapply(strsplit(v, "\\."), "[", 3), c(
  "unad", "forw", "back"), c("Base", "Forward Looking", "Backward Looking"))]
d[, Match.Type := factor(grepl("prim", v), c(TRUE, FALSE), c(
  "Primary", "Sensitivity"))]

# Reduce table
# Base & Backward Looking the same - keep Base
d[Adjustment == "Base", head1 := 1]
# Forward Looking different - keep Forward Looking
d[Adjustment == "Forward Looking", head1 := 2]
# Drop and format based on head1
d <- d[!is.na(head1)]
d[, head1 := factor(head1, 1:2, c("Base and Backward Looking", 
                                  "Forward Looking"))]
# Treatment data identical between primary and sensitivity - keep sensitivity
d[tc == "Treatment" & Match.Type == "Sensitivity", head2 := 1]
# Control data different - keep all
d[tc == "Control", head2 := 1 + as.numeric(Match.Type)]
# Drop and format based on head2
d <- d[!is.na(head2)]
d[, head2 := factor(head2, 1:3, c("Treatment", "Primary Control", 
                                  "Sensitivity Control"))] 

# Round population to 0 digits, comma format
d[grep("Population", variable), val := formatC(value, digits = 0, 
                                               big.mark = ",", format = "f")]

# Round percentages to 4 digits, comma format
d[grep("Population", variable, invert = TRUE), 
  val := sprintf("%.2f%%", value * 100)]

# Table
ident <- function(x) if (length(x) == 0) "" else x
(desc_stats <- tabular(
  Definition * Heading("Characteristic") * variable * Statistic ~ 
    Heading() * head1 * Heading() * head2 * Heading() * val * Heading() *
    ident, d))

# Write to external file
write.xlsx.tabular(desc_stats, file.path(appD, "Table 50.xlsx"))
