# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# State GDP ####

# Read file
state.gdp.file <- file.path(parent.dir, "Input Files", 
                            "BEA Gross State Product - 2011.xlsx")
state.gdp <- Read.xlsx2(state.gdp.file, sheetIndex = 1, 
                        colClasses = c(rep("character", 3), "numeric"))
setkey(state.gdp, fips)



# State business patterns ####

# Read file
st.cbp.file <- file.path(parent.dir, "Input Files", 
                         "Census Bureau State Business Patterns - 2011.xlsx")
st.cbp <- Read.xlsx2(st.cbp.file, sheetIndex = 1, 
                     colClasses = c("character", "numeric"))
setkey(st.cbp, state_fips)



# County business patterns ####

# Read file
cbp.file <- file.path(parent.dir, "Input Files", 
                      "Census Bureau County Business Patterns - 2011.xlsx")
cbp <- Read.xlsx2(cbp.file, sheetIndex = 1,
                  colClasses = c(rep("character", 2), rep("numeric", 3),
                                 "character", "numeric"))
setkey(cbp, state_fips, county_fips)

# "ERROR" is from the read procedure, just means missing in the data
cbp[flag == "ERROR", flag := NA_character_]
cbp[is.nan(midpoint), midpoint := NA_real_]

# Some CBP values are suppressed
# Use state values to interpolate

# How many employees are reported for each state at the county level?
st.cbp <- st.cbp[cbp[, list(disclosed = sum(pnf_employ)), by = state_fips]]

# How many to distribute among withheld counties?
st.cbp[, withheld := pnf_employ - disclosed]

# Add to county table
cbp[st.cbp, withheld := withheld]

# Sum of establishments of withheld counties
cbp[!is.na(flag), estsum := sum(establishments), by = state_fips]

# Withheld county / sum 
cbp[!is.na(flag), estprop := establishments / estsum]

# Apportion according to proportion
cbp[!is.na(flag), apportioned := estprop * withheld]
cbp[!is.na(flag), pnf_employ := pnf_employ + apportioned]

# Reduce columns
cbp <- cbp[, list(fips = paste0(state_fips, county_fips), pnf_employ, 
                  establishments, payroll)]
setkey(cbp, fips)



# County personal income ####

# Read file
county.pi.file <- file.path(parent.dir, "Input Files",
                            "BEA County Personal Income - 2011.xlsx")
county.pi <- Read.xlsx2(county.pi.file, sheetIndex = 1, 
                        colClasses = c("character", "numeric"))
setkey(county.pi, fips)

# Reported personal income is combined for some independent cities in VA
# Distribute reported combined values based on number of employed

# Table of combined areas
combined.areas <- as.data.table(melt(list(
  "51901" = c("Albemarle County, VA" = "51003", 
              "Charlottesville city, VA" = "51540"),
  "51903" = c("Alleghany County, VA" = "51005", "Covington city, VA" = "51580"),
  "51907" = c("Augusta County, VA" = "51015", "Staunton city, VA" = "51790", 
              "Waynesboro city, VA" = "51820"), 
  "51909" = c("Bedford County, VA" = "51019", "Bedford city, VA" = "51515"),
  "51911" = c("Campbell County, VA" = "51031", "Lynchburg city, VA" = "51680"),
  "51913" = c("Carroll County, VA" = "51035", "Galax city, VA" = "51640"),
  "51918" = c("Dinwiddie County, VA" = "51053", 
              "Colonial Heights city, VA" = "51570", 
              "Petersburg city, VA" = "51730"), 
  "51919" = c("Fairfax County, VA" = "51059", "Fairfax city, VA" = "51600", 
              "Falls Church city, VA" = "51610"), 
  "51921" = c("Frederick County, VA" = "51069",
              "Winchester city, VA" = "51840"),
  "51923" = c("Greensville County, VA" = "51081", "Emporia city, VA" = "51595"),
  "51929" = c("Henry County, VA" = "51089", "Martinsville city, VA" = "51690"),
  "51931" = c("James City County, VA" = "51095", 
              "Williamsburg city, VA" = "51830"),
  "51933" = c("Montgomery County, VA" = "51121", "Radford city, VA" = "51750"),
  "51939" = c("Pittsylvania County, VA" = "51143", 
              "Danville city, VA" = "51590"),
  "51941" = c("Prince George County, VA" = "51149",
              "Hopewell city, VA" = "51670"),
  "51942" = c("Prince William County, VA" = "51153",
              "Manassas city, VA" = "51683", 
              "Manassas Park city, VA" = "51685"),
  "51944" = c("Roanoke County, VA" = "51161", "Salem city, VA" = "51775"),
  "51945" = c("Rockbridge County, VA" = "51163", 
              "Buena Vista city, VA" = "51530", "Lexington city, VA" = "51678"), 
  "51947" = c("Rockingham County, VA" = "51165", 
              "Harrisonburg city, VA" = "51660"),
  "51949" = c("Southampton County, VA" = "51175",
              "Franklin city, VA" = "51620"),
  "51951" = c("Spotsylvania County, VA" = "51177", 
              "Fredericksburg city, VA" = "51630"),
  "51953" = c("Washington County, VA" = "51191", "Bristol city, VA" = "51520"),
  "51955" = c("Wise County, VA" = "51195", "Norton city, VA" = "51720"), 
  "51958" = c("York County, VA" = "51199", "Poquoson city, VA" = "51735"))))
setnames(combined.areas, names(combined.areas), c("fips", "pi_fips"))
setkey(combined.areas, fips)

# Percentage of employed in combined area
combined.areas <- cbp[combined.areas, list(pi_fips, pnf_employ)]
combined.areas[, combined_pnf_employ := sum(pnf_employ), keyby = pi_fips]
combined.areas[, combined_share := pnf_employ / combined_pnf_employ]
combined.areas[, c("pnf_employ", "combined_pnf_employ") := NULL]

# Split income of combined areas based on percentage
combined.areas.pi <- county.pi[combined.areas]
combined.areas.pi[, income := income * combined_share]
combined.areas.pi[, fips := fips.1][, c("fips.1", "combined_share") := NULL]

# Remove combined areas and and apportioned areas
county.pi <- rbind(county.pi[!fips %in% combined.areas[, pi_fips]], 
                   combined.areas.pi)



# Estimate GCP ####

# What is state personal income?
county.pi[, state := substr(fips, 1, 2)]
county.pi[, state_income := sum(income), keyby = state]

# How much of state's personal income is in county?
county.pi[, share := income / state_income]

# Gross county product (GCP) is county share of personal income * state GDP
gcp <- county.pi[state.gdp, list(fips, gcp = share * earnings)]

# Clean up table
gcp[, state := NULL]
setcolorder(gcp, c("fips", "gcp"))
setkey(gcp, fips)



# Save ####
gcp.rda <- file.path(parent.dir, "Prepared Data", "Gross County Product.rda")
save(gcp, file = gcp.rda)



# Write ####
gcp.xlsx <- file.path(parent.dir, "Prepared Data",
                      "Gross County Product.xlsx")
write.xlsx2(gcp, gcp.xlsx, row.names = FALSE)
