# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"),
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)



# Centers established and improved (APRs) ####

# Read file
apr.file <- file.path(parent.dir, "Input Files", "PCC Annual PPR Data.xlsx")
est_impr <- Read.xlsx2(apr.file, sheetIndex = 1, colClasses = c(
  rep("character", 2), rep("numeric", 18)))

# Reshape
est_impr <- melt(est_impr, id.vars = c("Award_Num", "Year_Desc"))
est_impr[, variable := as.character(variable)]

# Established/improved
est_impr[, v := factor(grepl("Establ", variable), c(TRUE, FALSE), 
                       c("established", "improved"))]

# Category
est_impr[, category := factor(sapply(strsplit(variable, "_"), "[", 1), c(
  "School", "Library", "CCollege", "Universt", "Healthcr", "PublSafety", 
  "EconInst", "OtherGov", "OtherNonGov"), c(
    "School (K-12)", "Library", "Community College", 
    "Universities and Colleges", "Medical or Healthcare Facility", 
    "Public Safety Entities", 
    "Job Training and/or Economic Development Institution",
    "Other Community Support (Governmental)",
    "Other Community Support (Non-Governmental)"))]

# Reshape
est_impr <- data.table(
  dcast(est_impr, Award_Num + Year_Desc + category ~ v, value.var = "value"),
  key = "Award_Num")

# Add year column as date
est_impr[, year := as.Date(paste0(Year_Desc, "-12-31"))]

# Value columns
val <- c("established", "improved")
est_impr[, val := Map(replace, .SD, lapply(.SD, is.na), 0), .SDcols = val, 
         with = FALSE]

# Did the grantee report data this year?
est_impr[, reported := Reduce("+", .SD) > 0, .SDcols = val]

# If grantee never reported any data, set all to TRUE
(noreport <- est_impr[, list(reported = any(reported)), 
                      keyby = Award_Num][!(reported)])
est_impr[noreport, reported := TRUE]

# Find first PPR with reported data for each grantee
est_impr[, first := min(year[reported]), by = Award_Num]
est_impr[, table(first)]

# Data are cumulative. Some grantees stopped reporting as grant ended.
est_impr[, last := max(year), by = Award_Num]
est_impr[, table(last)]

# Roll last available APR numbers to end
keys <- c("Award_Num", "category", "year")
# Every grant * every category * every year
all_all_all <- do.call("CJ", lapply(est_impr[, keys, with = FALSE], unique))
# Query every grant * every category * every year from data
# Roll over where missing
setkeyv(est_impr, keys)
est_impr <- est_impr[all_all_all, roll = TRUE, allow.cartesian = TRUE]
# Mark rolled observations
est_impr[, rolled := year > last]
# Drop observations before reporting started
est_impr <- est_impr[year >= first]
# Replace rolled over character values with correct ones based on date
est_impr[, Year_Desc := year(year)]
# Drop "reported", "first", and "last" columns
est_impr[, c("reported", "first", "last") := NULL]

# Key
setkey(est_impr, Award_Num, category)


# Save ####
centers.rda <- file.path(parent.dir, "Prepared Data",
                         "PCC Centers Established and Improved.rda")
save(est_impr, file = centers.rda)



# Write ####
centers.xlsx <- file.path(parent.dir, "Prepared Data", 
                          "PCC Centers Established and Improved.xlsx")
write.xlsx2(grants[est_impr], centers.xlsx, row.names = FALSE)
