# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code",
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)



# Installations and upgrades (PPRs) ####

# Read file
ppr.file <- file.path(parent.dir, "Input Files", "PCC Quarterly PPR Data.xlsx")
inst_upgr <- Read.xlsx2(ppr.file, sheetIndex = 1, colClasses = c(
  rep("character", 3), rep("numeric", 5)))
inst_upgr[, UserAverage_Num := NULL]
setkey(inst_upgr, Award_Num)

# Add quarter column as date
inst_upgr[, quarter := paste(Year_Desc,
                             as.numeric(substr(Quarter_Cd, 2, 2)) * 3, 1)]
inst_upgr[, quarter := as.Date(quarter, "%Y %m %d") + months(1) - days(1)]

# Value columns
val <- paste0(c("WorkstatnInstall", "ConnectUpgrade", "ConnectEstablish", 
                "PCCOpenAddHours"), "_Num")
inst_upgr[, val := Map(replace, .SD, lapply(.SD, is.na), 0), .SDcols = val, 
          with = FALSE]

# There is some duplication...
setkey(inst_upgr, Award_Num, quarter)
inst_upgr[unique(inst_upgr[duplicated(inst_upgr), SJ(Award_Num, quarter)])]

# ...but the values are identical
inst_upgr[unique(inst_upgr[duplicated(inst_upgr), SJ(Award_Num, quarter)])][
  , lapply(.SD, function(x) length(unique(x)) == 1), .SDcols = val, 
  by = list(Award_Num, quarter)]
setkey(inst_upgr, NULL)
inst_upgr <- unique(inst_upgr)
setkey(inst_upgr, Award_Num, quarter)

# Did the grantee report data this year?
inst_upgr[, reported := Reduce("+", .SD) > 0, .SDcols = val]

# If grantee never reported any data, set all to TRUE
(noreport <- inst_upgr[, list(reported = any(reported)), 
                       keyby = Award_Num][!(reported)])
inst_upgr[noreport, reported := TRUE]

# Find first PPR with reported data for each grantee
inst_upgr[, first := min(quarter[reported]), by = Award_Num]
inst_upgr[, table(first)]

# Data are cumulative. Some grantees stopped reporting as grant ended.
inst_upgr[, last := max(quarter), by = Award_Num]
inst_upgr[, table(last)]

# Roll last available PPR numbers to end
keys <- c("Award_Num", "quarter")
# Every grant * every quarter
all_all <- do.call("CJ", lapply(inst_upgr[, keys, with = FALSE], unique))
# Query every grant * every quarter from data
# Roll over where missing
setkeyv(inst_upgr, keys)
inst_upgr <- inst_upgr[all_all, roll = TRUE, allow.cartesian = TRUE]
# Mark rolled observations
inst_upgr[, rolled := quarter > last]
# Drop observations before reporting started
inst_upgr <- inst_upgr[quarter >= first]
# Replace rolled over character values with correct ones based on date
inst_upgr[, Year_Desc := year(quarter)]
inst_upgr[, Quarter_Cd := paste0("Q", quarter(quarter))]
# Drop "reported", "first", and "last" columns
inst_upgr[, c("reported", "first", "last") := NULL]



# Save ####
equipment.rda <- file.path(parent.dir, "Prepared Data", "PCC Equipment.rda")
save(inst_upgr, file = equipment.rda)



# Write ####
equipment.file <- file.path(parent.dir, "Prepared Data",
                            "PCC Equipment.xlsx")
write.xlsx2(grants[inst_upgr], equipment.file, row.names = FALSE)
