# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)



# Read reported training ####

# APR
apr.file <- file.path(parent.dir, "Input Files", 
                      "Training Activities - APR.xlsx")
apr <- Read.xlsx2(apr.file, sheetIndex = 1, colClasses = c(
  rep("character", 2), rep("numeric", 18), "character", rep("numeric", 10)))

# PCC PPR
pcc.file <- file.path(parent.dir, "Input Files",
                      "Training Activities - PCC.xlsx")
pcc <- Read.xlsx2(pcc.file, sheetIndex = 1, colClasses = c(
  rep("character", 4), rep("numeric", 5)))

# SBA PPR
sba.file <- file.path(parent.dir, "Input Files",
                      "Training Activities - SBA.xlsx")
sba <- Read.xlsx2(sba.file, sheetIndex = 1, colClasses = c(
  rep("character", 6), rep("numeric", 3)))



# Manipulate training tables ####

# Focus area vectors

## Focus area tags
fa <- c("DL", "WED", "ED", "HC", "QOL")

## Focus area participants column names
p <- paste0(fa, "_p")

## Focus area hours column names
h <- paste0(fa, "_h")

## Healthcare/quality of life tags
hq <- c("HC", "QOL")


# Multiply "Other" hours and participants by focus area weights
apr[, p := lapply(.SD, "*", apr[, OtherTrain_Particip_Num]), .SDcols = p, 
    with = FALSE]
apr[, h := lapply(.SD, "*", apr[, OtherTrain_TotHour_Num]), .SDcols = h,
    with = FALSE]

## Remove "Other" hours and participants that were put into 
## Focus areas 
apr[, OtherTrain_Particip_Num := OtherTrain_Particip_Num - 
      Reduce("+", apr[, p, with = FALSE])]
apr[, OtherTrain_TotHour_Num := OtherTrain_TotHour_Num - 
      Reduce("+", apr[, h, with = FALSE])]


# Reshape APR table
apr <- melt(apr, id.vars = c("Award_Num", "Year_Desc"), measure.vars = c(
    "Multimed_Particip_Num", "Multimed_TotHour_Num", "OffcSkill_Particip_Num", 
    "OffcSkill_TotHour_Num", "ESL_Particip_Num", "ESL_TotHour_Num", 
    "GED_Particip_Num", "GED_TotHour_Num", "ColPrep_Particip_Num", 
    "ColPrep_TotHour_Num", "Internet_Particip_Num", "Internet_TotHour_Num",
    "CertProg_Particip_Num", "CertProg_TotHour_Num", "OtherTrain_Particip_Num",
    "OtherTrain_TotHour_Num", "DL_p", "WED_p", "ED_p", "HC_p", "QOL_p", "DL_h",
    "WED_h", "ED_h", "HC_h", "QOL_h"))
apr[grep("Particip", variable), var := "p"]
apr[grep("TotHour", variable), var := "h"]
apr[grep("_p$", variable), var := "p"]
apr[grep("_h$", variable), var := "h"]
apr[, fa := sapply(strsplit(as.character(variable), "_"), "[", 1)]
apr[, variable := NULL]
apr[, source := "APR"]


# Map reported categories to focus areas
apr[, fa := factor(fa)]
levels(apr$fa)[levels(apr$fa) %in% c("Multimed", "Internet")] <- "DL"
levels(apr$fa)[levels(apr$fa) %in% "OffcSkill"] <- "WED"
levels(apr$fa)[levels(apr$fa) %in% c("ESL", "GED", "ColPrep", "CertProg")] <- 
  "ED"
levels(apr$fa)[levels(apr$fa) %in% "OtherTrain"] <- "O"


# Create wide APR table - will be combined with PPR data
apr.ruler <- Dcast(apr, Award_Num + Year_Desc + var ~ source + fa, 
                   value.var = "value", 
                   fun.aggregate = function(...) sum(..., na.rm = TRUE))


# PPR tables
setnames(pcc, c("Training_Programs_Hours", "Training_Programs_Participation"), 
         c("h", "p"))
setnames(sba, "ParticipActl_Num", "p")

## SBA hours = 7 * SBA participants
sba[, h := p * 7]

## Reshape to combine with APR
ids <- "Award_Num"
ids2 <- c(ids, "Year_Desc", "Quarter_Cd", "HC", "QOL")
measures <- c("h", "p")
ppr.ruler <- as.data.table(rbind(
  melt(pcc, id.vars = ids2, measure.vars = measures),
  melt(sba, id.vars = ids2, measure.vars = measures)))
setnames(ppr.ruler, "variable", "var")
ppr.ruler[, PPR_HC := HC * value]
ppr.ruler[, PPR_QOL := QOL * value]
ppr.ruler <- ppr.ruler[, lapply(.SD, sum), .SDcols = c("PPR_HC", "PPR_QOL"), 
                       by = c(ids, "var")]


# Merge APR & PPR
to.rule <- merge(apr.ruler, ppr.ruler, by = c(ids, "var"), all = TRUE)
to.rule[, which(sapply(to.rule, is.numeric)) := 
          Map(replace, .SD, lapply(.SD, is.na), 0),
        .SDcols = which(sapply(to.rule, is.numeric))]



# Adjustments

# 06-43-B10584: University of California, Davis, California Telehealth
# Network
# Everything in APR is healthcare. PPR totals are greater than APR totals.
# Set PPR totals to 0.
to.rule[J("06-43-B10584"), PPR_HC := 0]

# 06-43-B10594: City and County of San Francisco, SF-CBOP
# They reported website hits, which shouldn't be classified as training
to.rule[Award_Num == "06-43-B10594" & var == "p", PPR_HC := PPR_HC - 
          sba[Award_Num == "06-43-B10594"][
            Activity_Description == "WWW SFHEALTHYAGING ORG"][, sum(p)]]
to.rule[Award_Num == "06-43-B10594" & var == "h", PPR_HC := PPR_HC - 
          sba[Award_Num == "06-43-B10594"][
            Activity_Description == "WWW SFHEALTHYAGING ORG"][, sum(h)]]
# One of the QOL's is 16,000 reported, probably website hits
to.rule[Award_Num == "06-43-B10594" & var == "p", PPR_QOL := PPR_QOL - 
          sba[Award_Num == "06-43-B10594"][
            Activity_Description == "INTRODUCTION TO THE VOTER TOOLKIT WEBSITE AT HTTP SFELECTIONS ORG TOOLS VOTERKIT"][
              , sum(p)]]
to.rule[Award_Num == "06-43-B10594" & var == "h", PPR_QOL := PPR_QOL - sba[
  Award_Num == "06-43-B10594"][
    Activity_Description == 
      "INTRODUCTION TO THE VOTER TOOLKIT WEBSITE AT HTTP SFELECTIONS ORG TOOLS VOTERKIT"][
        , sum(h)]]

# 13-43-B10576: Georgia Partnership for TeleHealth, Inc., TeleConnect
# Georgia for Better Health
# All reported "Other" are HC, ignore PPR
to.rule[J("13-43-B10576"), PPR_HC := 0]

# 24-42-B10017: Coppin State University, Coppin Heights-Rosemont 
# Family Computer Center
# Reported zeros in APR, use latest PPR data
coppin <- pcc[Award_Num == "24-42-B10017"]
coppin[, c("DL", "WED", "ED", "O") := list(0, 0, 0, 0)]
coppin[grepl("COMPUTERS", Training_Programs_Name), DL := 1]
coppin[grepl("INTERNET", Training_Programs_Name), DL := 1]
coppin[grepl("MICROSOFT", Training_Programs_Name), DL := 1]
coppin[grepl("FILMING", Training_Programs_Name), DL := 1]
coppin[grepl("ART AND TECHNOLOGY", Training_Programs_Name), DL := 1]
coppin[grepl("EXCEL", Training_Programs_Name), DL := 1]
coppin[grepl("POWERPOINT", Training_Programs_Name), DL := 1]
coppin[grepl("JOB", Training_Programs_Name), WED := 1]
coppin[grepl("GED", Training_Programs_Name), ED := 1]
coppin[grepl("CLASSROOM", Training_Programs_Name), ED := 1]
coppin[coppin[, Reduce("+", .SD), .SDcols = fa] == 0, O := 1]
coppin <- melt(coppin, id.vars = c(
  "Award_Num", "HC", "QOL", "DL", "WED", "ED", "O"), measure.vars = c("p", "h"))
coppin[, c(fa, "O") := lapply(.SD, "*", coppin[, value]), .SDcols = c(fa, "O")]
coppin <- coppin[, lapply(.SD, sum), by = c("Award_Num", "variable"), 
                 .SDcols = c(fa, "O")]
setnames(coppin, c("variable", fa, "O"), c("var", paste0("APR_", c(fa, "O"))))
coppin[, c("PPR_HC", "PPR_QOL") := list(0, 0)]
to.rule <- as.data.table(rbind.fill(to.rule[!J("24-42-B10017")], coppin))
setkey(to.rule, Award_Num)

# 36-43-B10559: Wildwood Programs, Inc., Broadband Video for Human
# Services Agencies
# No DL to take HC out of, move WED over to DL and continue as usual
to.rule[J("36-43-B10559"), APR_DL := PPR_HC]
to.rule[J("36-43-B10559"), APR_WED := APR_WED - PPR_HC]



# Apply rules from focus area training memo ####

# 3a. i. If the grantee's Annual PPR Healthcare total is greater than the
# PCC or SBA Healthcare PPR total, no action is taken.
to.rule[APR_HC >= PPR_HC, PPR_HC := 0]

# 3a. ii. If the grantee's Annual PPR Healthcare total is less than the
# PCC or SBA Healthcare PPR total, the difference in hours is removed from
# the grantee's remaining "Other" Annual PPR hours and placed in 
# Healthcare. If "Other" does not have sufficient hours, the remainder is
# removed from Digital Literacy and placed in Healthcare.
to.rule[APR_HC < PPR_HC, PPR_HC := PPR_HC - APR_HC]
to.rule[, HC := APR_HC + PPR_HC]
to.rule[, APR_O := APR_O - PPR_HC]
to.rule[, PPR_HC := 0]
to.rule[APR_O < 0, APR_DL := APR_DL + APR_O]
to.rule[APR_O < 0, APR_O := 0]

# 3b. i. If the grantee's Annual PPR Quality of Life / Civic Engagement
# total is greater than the PCC or SBA Quality of Life / Civic Engagement
# PPR total, no action is taken.
to.rule[APR_QOL >= PPR_QOL, PPR_QOL := 0]

# 3b. ii. If the grantee's Annual PPR Quality of Life / Civic Engagement
# total is less than the PCC or SBA Quality of Life / Civic Engagement PPR
# total, the difference in hours is removed from the grantee's remaining
# "Other" Annual PPR hours and placed in Quality of Life / Civic 
# Engagement. If "Other" does not have sufficient hours, the remainder is
# removed from Digital Literacy and placed in Quality of Life / Civic
# Engagement.
to.rule[APR_QOL < PPR_QOL, PPR_QOL := PPR_QOL - APR_QOL]
to.rule[, QOL := APR_QOL + PPR_QOL]
to.rule[, APR_O := APR_O - PPR_QOL]
to.rule[, PPR_QOL := 0]
to.rule[APR_O < 0, APR_DL := APR_DL + APR_O]
to.rule[APR_O < 0, APR_O := 0]

# 3c. i. Changes to APR data are made using hourly figures. PCCs report
# training in terms of training hours, while SBAs report training
# participants. Using the quarterly PCC training data, an average of 7
# hours per training activity was calculated. SBA training participant
# figures are multiplied by 7 to arrive at training hours.  For example,
# if 10 SBA participants need to be moved from "Other" to Healthcare, 70
# training hours is used.
# note: "* 7" done in reshaping steps

# ruled table
training <- to.rule[, list(Award_Num, var = factor(var, c("p", "h"), c(
  "Participants", "Hours")), DL = APR_DL, ED = APR_ED, WED = APR_WED, HC, QOL,
  O = APR_O)]



# Get lab hours/participants from APR file
lab <- Read.xlsx2(apr.file, sheetIndex = 1, colClasses = c(
  rep("character", 2), rep("numeric", 18), "character", rep("numeric", 10)))
lab <- lab[, list(Award_Num, Participants = LabAccess_Particip_Num, 
                  Hours = LabAccess_TotHour_Num)]


# Combine lab and training
ids <- intersect(names(training), names(lab))
training <- as.data.table(melt(training, id.vars = c(ids, "var")))
lab <- as.data.table(melt(lab, id.vars = ids, variable.name = "var"))
lab[, variable := "LAB"]
training.lab <- as.data.table(rbind.fill(training, lab))
training.lab[, variable := factor(variable, c(
  "WED", "ED", "HC", "QOL", "DL", "O", "LAB"), c(
    "Workforce and Economic Development", "Education and Training", 
    "Healthcare", "Quality of Life/Civic Engagement", "Digitial Literacy", 
    "Other", "Open Lab"))]
training.lab <- Dcast(training.lab, Award_Num + variable ~ var)
setnames(training.lab, "variable", "Focus Area")
setkey(training.lab, Award_Num)



# Save to file ####
training.rda <- file.path(parent.dir, "Prepared Data", 
                          "PCC and SBA Training Hours and Participants.rda")
save(training.lab, file = training.rda)



# Write ####
training.xlsx <- file.path(parent.dir, "Prepared Data", 
                           "PCC and SBA Training Hours and Participants.xlsx")
write.xlsx2(grants[training.lab], training.xlsx, row.names = FALSE)
