# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)

# PCC and SBA Average Household Size
hhsize.rda <- file.path(parent.dir, "Prepared Data", 
                        "PCC and SBA Average Household Size.rda")
load(hhsize.rda)
head(avg_hh_size)



# Subscribers data ####

# Read subscribers file 
ppr.file <- file.path(parent.dir, "Input Files", "SBA Quarterly PPR Data.xlsx")
sba_subscribers <- Read.xlsx2(ppr.file, sheetIndex = 1)

# Make columns numeric
num <- c("Year_Desc", "SubscrNewHouseTot_Num", "SubscrNewBusinessTot_Num")
sba_subscribers[, num := lapply(.SD, as.numeric), .SDcols = num, with = FALSE]

# Key
setkey(sba_subscribers, Award_Num, Year_Desc, Quarter_Cd)



# Fix data issues ####
# 39 grantee * year * quarter combinations have multiple entries
# proof: unique(sba_subscribers[duplicated(sba_subscribers)])

# If data aren't identical, use online PPRs to pick correct values
sba_subscribers[J('06-43-B10538', 2011, 'Q3'), 4:5 := list(166, 5)]
sba_subscribers[J('06-43-B10540', 2011, 'Q3'), 4:5 := list(0, 0)]
sba_subscribers[J('06-43-B10594', 2013, 'Q1'), 5 := 51]
sba_subscribers[J('06-43-B10594', 2013, 'Q2'), 5 := 51]
sba_subscribers[J('17-43-B10507', 2012, 'Q1'), 4:5 := list(61, 94)]
sba_subscribers[J('21-43-B10546', 2011, 'Q4'), 5 := 315]
sba_subscribers[J('21-43-B10546', 2012, 'Q1'), 4:5 := list(2503, 90)]
sba_subscribers[J('21-43-B10546', 2013, 'Q4'), 4:5 := list(30425, 2209)] 
                # ^ no online PPR, most sensible of reported values
sba_subscribers[J('26-43-B10564', 2012, 'Q1'), 4:5 := list(36, 0)]
sba_subscribers[J('42-43-B10571', 2011, 'Q4'), 4:5 := list(2, 1)]
sba_subscribers[J('42-43-B10571', 2012, 'Q1'), 4:5 := list(10, 1)]
sba_subscribers[J('42-43-B10571', 2012, 'Q2'), 4:5 := list(25, 1)]
sba_subscribers[J('42-43-B10571', 2012, 'Q3'), 4:5 := list(241, 2)]
sba_subscribers[J('42-43-B10571', 2012, 'Q4'), 4:5 := list(545, 2)]
sba_subscribers[J('42-43-B10571', 2013, 'Q1'), 4:5 := list(1468, 39)]
sba_subscribers[J('42-43-B10571', 2013, 'Q3'), 4:5 := list(5276, 50)]
sba_subscribers[J('47-43-B10542', 2012, 'Q3'), 4:5 := list(13535, 9303)]
sba_subscribers[J('55-43-B10539', 2012, 'Q2'), 4:5 := list(12150, 720)]
sba_subscribers[J('55-43-B10539', 2012, 'Q4'), 4:5 := list(13798, 980)]

# Drop duplicate entries
setkey(sba_subscribers, NULL)
sba_subscribers <- unique(sba_subscribers)
setkey(sba_subscribers, Award_Num)


# Manipulate data ####

# Add quarter column as date
sba_subscribers[, quarter := paste(
  Year_Desc, as.numeric(substr(Quarter_Cd, 2, 2)) * 3, 1)]
sba_subscribers[, quarter := as.Date(quarter, "%Y %m %d") + months(1) - days(1)]

# Rename to show cumulative
setnames(sba_subscribers, c(
  "SubscrNewHouseTot_Num", "SubscrNewBusinessTot_Num"), 
  c("hhold_cum", "bus_cum"))

# Value columns
val <- c("hhold_cum", "bus_cum")
sba_subscribers[, val := Map(replace, .SD, lapply(.SD, is.na), 0), 
                .SDcols = val, with = FALSE]

# Did the grantee report data this quarter?
sba_subscribers[, reported := Reduce("+", .SD) > 0, .SDcols = val]

# If grantee never reported any data, set all to TRUE
(noreport <- sba_subscribers[, list(reported = any(reported)),
                             keyby = Award_Num][!(reported)])
sba_subscribers[noreport, reported := TRUE]

# Find first PPR with reported data for each grantee
sba_subscribers[, first := min(quarter[reported]), by = Award_Num]
sba_subscribers[, table(first)]

# PPRs are cumulative. Some grantees stopped reporting as grant ended.
sba_subscribers[, last := max(quarter), by = Award_Num]
sba_subscribers[, table(last)]

# Roll last PPR subscriber numbers to end
keys <- c("Award_Num", "quarter")
# Every grant * every quarter
all_all <- do.call("CJ", lapply(sba_subscribers[, keys, with = FALSE], unique))
# Query every grant * every quarter from data
# Roll over where missing
setkeyv(sba_subscribers, keys)
sba_subscribers <- sba_subscribers[all_all, roll = TRUE]
# Mark rolled observations
sba_subscribers[, rolled := quarter > last]
# Drop observations before reporting started
sba_subscribers <- sba_subscribers[quarter >= first]
# Replace rolled over character values with correct ones based on date
sba_subscribers[, Year_Desc := year(quarter)]
sba_subscribers[, Quarter_Cd := paste0("Q", quarter(quarter))]
# Drop "reported", "first", and "last" columns
sba_subscribers[, c("reported", "first", "last") := NULL]

# PPRs are cumulative. Difference to get new in each quarter.
setkey(sba_subscribers, Award_Num, quarter)
sub <- c("hhold_cum", "bus_cum")
sba_subscribers[, sub := Map(replace, .SD, lapply(.SD, is.na), 0), 
                .SDcols = sub, with = FALSE]
sba_subscribers[, c("hhold_quarter", "bus_quarter") := lapply(
  .SD, function(x) diff(c(0, x))), .SDcols = sub, by = Award_Num]

# PPRs report households, not people. Multiply households times average 
# household size.
sba_subscribers[avg_hh_size, `:=` (
  avg_hh_size = avg_hh_size, subscr_cum = avg_hh_size * hhold_cum, 
  subscr_quarter = avg_hh_size * hhold_quarter)]

# Totals table - last quarter
sba_subscr_total <- sba_subscribers[Year_Desc == 2013 & Quarter_Cd == "Q4"]
sba_subscr_total[, c("hhold_quarter", "bus_quarter", "subscr_quarter", "rolled",
                     "Year_Desc", "Quarter_Cd", "quarter") := NULL]
setkey(sba_subscr_total, Award_Num)



# Save ####
subscr.rda <- file.path(parent.dir, "Prepared Data", "SBA Subscribers.rda")
save(sba_subscr_total, sba_subscribers, file = subscr.rda)



# Write ####
subscr.xlsx <- file.path(parent.dir, "Prepared Data", "SBA Subscribers.xlsx")
write.xlsx2(grants[sba_subscr_total], subscr.xlsx, "Total Subscribers",
            row.names = FALSE)
write.xlsx2(grants[sba_subscribers], subscr.xlsx, "Quarterly Subscribers", 
            row.names = FALSE, append = TRUE)