# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), echo = TRUE)



# Load analysis ####

# Matching
matching.rda <- file.path(parent.dir, "Statistical Analysis", "Matching.rda")
load(matching.rda)
head(match.data)



# Load Prepared Data ####

# Treatment and Potential Control Counties
treat.control.rda <- file.path(parent.dir, "Prepared Data", 
                               "Treatment and Potential Control Counties.rda")
load(treat.control.rda)
head(btop)



# Functions ####

# Compare overall availability
reshapeMatched <- function(matches, avail11, avail13, 
                           pop = c("pop_total_2011_06", "pop_total_2013_06"),
                           data = match.data) {  
  
  av <- gsub("ppop", "pop", c(avail11, avail13))
  v <- c("fips", av, pop)
  m <- copy(get(matches))
  dk <- key(data)
  setkey(data, fips)
  
  # Data
  setkey(m, treat)
  tr <- data[m, v, with = FALSE][, treat := fips]
  setkey(m, control)
  co <- data[m][, c("treat", v), with = FALSE]
  
  # Reshape
  tr <- melt(tr, id.vars = c("treat", "fips"))
  co <- melt(co, id.vars = c("treat", "fips"))
  
  # Tag
  tr[, tc := 1]
  co[, tc := 2]
  
  # Combine
  d <- rbind(tr, co)
  d <- d[, list(value = sum(value)), by = list(variable, tc)]
  
  # Reshape
  d[, Release := gsub(".*([[:digit:]]{4})_([[:digit:]]{2})", "\\1-\\2-01", variable)]
  d[, Release := as.Date(Release) + months(1) - days(1)]
  d[, var := factor(variable %in% pop, c(TRUE, FALSE), c("pop", "bb"))]
  d <- Dcast(d, Release + tc ~ var, value.var = "value")
  
  # Rates
  d[, perc := bb / pop]
  d <- melt(d, id.vars = 1:2, measure.vars = "perc")
  d[, tc := factor(tc, labels = c("Treatment", "Control"))]
  d <- Dcast(d, Release ~ tc, value.var = "value")
  
  # Reset
  setkeyv(data, dk)
  
  # Return
  d[, matches := matches]
  setkey(d, Release)
  d
  
}

# Turn compound variables into better descriptors
tagResults <- function(d) {
  d[, definition := factor(toupper(sapply(
    strsplit(matches, "\\."), "[", 2)), c("NOFA", "NTIA"))]
  d[, adjustment := factor(sapply(
    strsplit(matches, "\\."), "[", 3), c("unad", "forw", "back"), c(
      "Availability", "Forward Looking", "Backward Looking"))]
  d[, match.type := factor(sapply(
    strsplit(matches, "\\."), "[", 4), c("prim", "sens"), 
    c("Primary", "Sensitivity"))]
  d[, matches := NULL]
}

# Calculate difference-in-differences for reshaped data
did <- function(data) {
  
  # Data
  d <- copy(data)
  k <- key(d)
  
  # Reshape
  dlong <- melt(d, id.vars = k, variable.name = "tc", value.name = "rate")
  
  # Differences
  dlong[, Difference := diff(rate[order(Release)]), by = c("tc", setdiff(k, "Release"))]
  
  # Reshape
  d2 <- Dcast(unique(dlong[, c(k, "tc", "Difference"), with = FALSE]), 
              as.formula(paste(paste(k, collapse = "+"), "~ tc")), value.var = "Difference")
  setnames(d, c("Control", "Treatment"), c("Control Rate", "Treatment Rate"))
  setnames(d2, c("Control", "Treatment"), c("Control Difference", "Treatment Difference"))
  d <- merge(d, d2, by = intersect(names(d), names(d2)))
  
  # Differences-in-differences
  d[, "Difference-in-Differences" := `Treatment Difference` - `Control Difference`]
  
  # Refactor
  v <- names(data)[sapply(data, is.factor)]
  d[, v := Map(factor, .SD, levels = lapply(data[, v, with = FALSE], levels)), 
    .SDcols = v, with = FALSE]
  
  # Return
  setkeyv(d, k)
  setcolorder(d, unique(c(k, "Treatment Rate", "Control Rate", "Treatment Difference",
                          "Control Difference", "Difference-in-Differences")))
  d
  
}



# Reshape match results ####

# List of matching results objects
match.list <- c("m.nofa.back.prim", "m.nofa.back.sens", "m.nofa.forw.prim", "m.nofa.forw.sens",
                "m.nofa.unad.prim", "m.nofa.unad.sens", "m.ntia.back.prim", "m.ntia.back.sens", 
                "m.ntia.forw.prim", "m.ntia.forw.sens", "m.ntia.unad.prim", "m.ntia.unad.sens")

# Combine results into a list
matches <- mget(match.list, .GlobalEnv)

# Which availability rates were used?
avail11 <- sapply(matches, attr, "availability")

# Which availability rates will be differenced?
avail13 <- gsub("2011", "2013", avail11)

# Availability rates for all matches
overall <- Map(reshapeMatched, match.list, avail11, avail13)
overall <- rbindlist(overall)
tagResults(overall)
setkey(overall, definition, adjustment, match.type, Release)



# Calculate difference-in-differences ####
did.overall <- did(overall)



# Effects table ####

# DIDs
effects <- did.overall[Release == "2013-06-30"]
effects[, c("Release", "Treatment Rate", "Control Rate", "Treatment Difference", 
            "Control Difference") := NULL]

# Add population to effects
pop <- match.data[btop][, list(fips, case, pop = pop_total_2013_06)]
pop <- dcast(pop, . ~ case, value.var = "pop", margins = "case", fun.aggregate = sum)[-1]
k <- c("Evaluation Study Sample", "Rest of BTOP", "All BTOP")
effects[, k := pop, with = FALSE]
effects[, k := lapply(.SD, "*", effects[, `Difference-in-Differences`]), .SDcols = k, with = FALSE]



# Resampling to get confidence intervals of difference-in-differences ####

# Resampling function
resampleEffect <- function(matches, avail11, avail13, seed, data = match.data) {
  
  # Print which set of matches and variables used
  cat(matches, avail11, avail13, "\n")
  
  # Get table of matches
  m <- copy(get(matches))
  
  # Copy data to preserve integrity
  d <- copy(data)
  setkey(d, fips)
  setnames(d, gsub("ppop", "pop", c(avail11, avail13)), c("avail11", "avail13"))
  setnames(d, c("pop_total_2011_06", "pop_total_2013_06"), c("pop11", "pop13"))
  
  # Lookup treatment county values
  setkey(m, treat)
  tr <- d[m][, list(treat = fips, control, avail11, avail13, pop11, pop13)]
  
  # Lookup control county values
  setkey(m, control)
  co <- d[m][, list(treat, control = fips, avail11, avail13, pop11, pop13)]
  
  # Merge treatment and control values
  setnames(tr, names(tr)[-1:-2], paste0("tr_", names(tr)[-1:-2]))
  setnames(co, names(co)[-1:-2], paste0("co_", names(co)[-1:-2]))
  tc <- merge(tr, co, by = c("treat", "control")) 
  
  # Function to calculate effect for i row numbers
  didfun <- function(tc, i, ...) {
    x <- tc[i, lapply(.SD, sum), .SDcols = 3:ncol(tc)]
    (x[, tr_avail13 / tr_pop13] - x[, tr_avail11 / tr_pop11]) - 
      (x[, co_avail13 / co_pop13] - x[, co_avail11 / co_pop11])
  }
  
  # Resample
  set.seed(seed)
  b <- boot(data = tc, statistic = didfun, R = 1000, sim = "ordinary")
  
  # Get confidence interval
  ci <- boot.ci(b, type = "basic")
  
  # Return table with effect, lower and upper limits, and individual replication values
  data.table(effect = b$t0, lower = ci$basic[1, 4], upper = ci$basic[1, 5], 
             replicates = b$t[, 1], matches = matches)
  
}

# Get seeds for each resample
set.seed(8675309)
seeds <- sample(10000:99999, length(match.list))

# Resample every effect estimate
resampled.ci <- Map(resampleEffect, match.list, avail11, avail13, seeds)
resampled.ci <- rbindlist(resampled.ci)
tagResults(resampled.ci)
setkey(resampled.ci, definition, adjustment, match.type)

# Table of effects and intervals for every estimate
effects.intervals <- unique(resampled.ci)
effects.intervals[, replicates := NULL]

# Table of replicates for base case
base.replications <- resampled.ci[J("NTIA", "Availability", "Primary")]



# Save ####
effects.rda <- file.path(parent.dir, "Statistical Analysis", "Effect on Availability.rda")
save(effects, effects.intervals, base.replications, did.overall, file = effects.rda)



# Write ####
effects.xlsx <- file.path(parent.dir, "Statistical Analysis", "Effect on Availability.xlsx")
writeIt <- function(data, sheet, append = TRUE) {
  write.xlsx2(data, effects.xlsx, sheet, row.names = FALSE, append = append)
}
writeIt(effects, "Effects", append = FALSE)
writeIt(effects.intervals, "Effects Intervals")
writeIt(base.replications, "Base Case Resamples")
writeIt(did.overall, "Overall Estimates")
